@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2005 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2017 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,10 +12,10 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
-"""Provide a password-interface wrapper around private archives.
-"""
+"""Provide a password-interface wrapper around private archives."""
 
 import os
 import sys
@@ -67,10 +67,17 @@ def main():
         return
 
     path = os.environ.get('PATH_INFO')
+    tpath = true_path(path)
+    if tpath <> path[1:]:
+        msg = _('Private archive - "./" and "../" not allowed in URL.')
+        doc.SetTitle(msg)
+        doc.AddItem(Header(2, msg))
+        print doc.Format()
+        syslog('mischief', 'Private archive hostile path: %s', path)
+        return
     # BAW: This needs to be converted to the Site module abstraction
     true_filename = os.path.join(
-        mm_cfg.PRIVATE_ARCHIVE_FILE_DIR,
-        true_path(path))
+        mm_cfg.PRIVATE_ARCHIVE_FILE_DIR, tpath)
 
     listname = parts[0].lower()
     mboxfile = ''
@@ -101,16 +108,27 @@ def main():
         msg = _('No such list <em>%(safelistname)s</em>')
         doc.SetTitle(_("Private Archive Error - %(msg)s"))
         doc.AddItem(Header(2, msg))
+        # Send this with a 404 status.
+        print 'Status: 404 Not Found'
         print doc.Format()
-        syslog('error', 'No such list "%s": %s\n', listname, e)
+        syslog('error', 'private: No such list "%s": %s\n', listname, e)
         return
 
     i18n.set_language(mlist.preferred_language)
     doc.set_language(mlist.preferred_language)
 
     cgidata = cgi.FieldStorage()
-    username = cgidata.getvalue('username', '')
-    password = cgidata.getvalue('password', '')
+    try:
+        username = cgidata.getfirst('username', '')
+    except TypeError:
+        # Someone crafted a POST with a bad Content-Type:.
+        doc.AddItem(Header(2, _("Error")))
+        doc.AddItem(Bold(_('Invalid options to CGI script.')))
+        # Send this with a 400 status.
+        print 'Status: 400 Bad Request'
+        print doc.Format()
+        return
+    password = cgidata.getfirst('password', '')
 
     is_auth = 0
     realname = mlist.real_name
@@ -124,14 +142,52 @@ def main():
         if cgidata.has_key('submit'):
             # This is a re-authorization attempt
             message = Bold(FontSize('+1', _('Authorization failed.'))).Format()
+            # give an HTTP 401 for authentication failure
+            print 'Status: 401 Unauthorized'
+        # Are we processing a password reminder from the login screen?
+        if cgidata.has_key('login-remind'):
+            if username:
+                message = Bold(FontSize('+1', _("""If you are a list member,
+                          your password has been emailed to you."""))).Format()
+            else:
+                message = Bold(FontSize('+1',
+                                _('Please enter your email address'))).Format()
+            if mlist.isMember(username):
+                mlist.MailUserPassword(username)
+            elif username:
+                # Not a member
+                if mlist.private_roster == 0:
+                    # Public rosters
+                    safeuser = Utils.websafe(username)
+                    message = Bold(FontSize('+1',
+                                  _('No such member: %(safeuser)s.'))).Format()
+                else:
+                    syslog('mischief',
+                       'Reminder attempt of non-member w/ private rosters: %s',
+                       username)
         # Output the password form
         charset = Utils.GetCharSet(mlist.preferred_language)
         print 'Content-type: text/html; charset=' + charset + '\n\n'
-        while path and path[0] == '/':
-            path=path[1:]  # Remove leading /'s
+        # Put the original full path in the authorization form, but avoid
+        # trailing slash if we're not adding parts.  We add it below.
+        action = mlist.GetScriptURL('private', absolute=1)
+        if mboxfile:
+            action += '.mbox'
+        if parts[1:]:
+            action = os.path.join(action, SLASH.join(parts[1:]))
+        # If we added '/index.html' to true_filename, add a slash to the URL.
+        # We need this because we no longer add the trailing slash in the
+        # private.html template.  It's always OK to test parts[-1] since we've
+        # already verified parts[0] is listname.  The basic rule is if the
+        # post URL (action) is a directory, it must be slash terminated, but
+        # not if it's a file.  Otherwise, relative links in the target archive
+        # page don't work.
+        if true_filename.endswith('/index.html') and parts[-1] <> 'index.html':
+            action += SLASH
+        # Escape web input parameter to avoid cross-site scripting.
         print Utils.maketext(
             'private.html',
-            {'action'  : mlist.GetScriptURL('private', absolute=1),
+            {'action'  : Utils.websafe(action),
              'realname': mlist.real_name,
              'message' : message,
              }, mlist=mlist)
@@ -159,6 +215,7 @@ def main():
         msg = _('Private archive file not found')
         doc.SetTitle(msg)
         doc.AddItem(Header(2, msg))
+        print 'Status: 404 Not Found'
         print doc.Format()
         syslog('error', 'Private archive file not found: %s', true_filename)
     else:
