@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2003 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -12,10 +12,12 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 """Produce and handle the member options."""
 
+import re
 import sys
 import os
 import cgi
@@ -31,9 +33,14 @@ from Mailman import MemberAdaptor
 from Mailman import i18n
 from Mailman.htmlformat import *
 from Mailman.Logging.Syslog import syslog
+from Mailman.CSRFcheck import csrf_check
 
+OR = '|'
 SLASH = '/'
 SETLANGUAGE = -1
+DIGRE = re.compile(
+    '<!--Start-Digests-Delete-->.*<!--End-Digests-Delete-->',
+     re.DOTALL)
 
 # Set up i18n
 _ = i18n._
@@ -45,12 +52,26 @@ except NameError:
     True = 1
     False = 0
 
+AUTH_CONTEXTS = (mm_cfg.AuthListAdmin, mm_cfg.AuthSiteAdmin,
+                 mm_cfg.AuthListModerator, mm_cfg.AuthUser)
 
 
 def main():
     doc = Document()
     doc.set_language(mm_cfg.DEFAULT_SERVER_LANGUAGE)
 
+    method = Utils.GetRequestMethod()
+    if method.lower() not in ('get', 'post'):
+        title = _('CGI script error')
+        doc.SetTitle(title)
+        doc.AddItem(Header(2, title))
+        doc.addError(_('Invalid request method: %(method)s'))
+        doc.AddItem('<hr>')
+        doc.AddItem(MailmanLogo())
+        print 'Status: 405 Method Not Allowed'
+        print doc.Format()
+        return
+
     parts = Utils.GetPathPieces()
     lenparts = parts and len(parts)
     if not parts or lenparts < 1:
@@ -77,36 +98,71 @@ def main():
         doc.addError(_('No such list <em>%(safelistname)s</em>'))
         doc.AddItem('<hr>')
         doc.AddItem(MailmanLogo())
+        # Send this with a 404 status.
+        print 'Status: 404 Not Found'
         print doc.Format()
-        syslog('error', 'No such list "%s": %s\n', listname, e)
+        syslog('error', 'options: No such list "%s": %s\n', listname, e)
         return
 
     # The total contents of the user's response
     cgidata = cgi.FieldStorage(keep_blank_values=1)
 
+    # CSRF check
+    safe_params = ['displang-button', 'language', 'email', 'password', 'login',
+                   'login-unsub', 'login-remind', 'VARHELP', 'UserOptions']
+    try:
+        params = cgidata.keys()
+    except TypeError:
+        # Someone crafted a POST with a bad Content-Type:.
+        doc.AddItem(Header(2, _("Error")))
+        doc.AddItem(Bold(_('Invalid options to CGI script.')))
+        # Send this with a 400 status.
+        print 'Status: 400 Bad Request'
+        print doc.Format()
+        return
+
+    if set(params) - set(safe_params):
+        csrf_checked = csrf_check(mlist, cgidata.getfirst('csrf_token'))
+    else:
+        csrf_checked = True
+    # if password is present, void cookie to force password authentication.
+    if cgidata.getfirst('password'):
+        os.environ['HTTP_COOKIE'] = ''
+        csrf_checked = True
+
     # Set the language for the page.  If we're coming from the listinfo cgi,
     # we might have a 'language' key in the cgi data.  That was an explicit
     # preference to view the page in, so we should honor that here.  If that's
     # not available, use the list's default language.
-    language = cgidata.getvalue('language')
+    language = cgidata.getfirst('language')
     if not Utils.IsLanguage(language):
         language = mlist.preferred_language
     i18n.set_language(language)
     doc.set_language(language)
 
     if lenparts < 2:
-        user = cgidata.getvalue('email')
+        user = cgidata.getfirst('email')
         if not user:
             # If we're coming from the listinfo page and we left the email
-            # address field blank, it's not an error.  listinfo.html names the
-            # button UserOptions; we can use that as the descriminator.
-            if not cgidata.getvalue('UserOptions'):
+            # address field blank, it's not an error.  Likewise if we're
+            # coming from anywhere else. Only issue the error if we came
+            # via one of our buttons.
+            if (cgidata.getfirst('login') or cgidata.getfirst('login-unsub')
+                    or cgidata.getfirst('login-remind')):
                 doc.addError(_('No address given'))
             loginpage(mlist, doc, None, language)
             print doc.Format()
             return
     else:
         user = Utils.LCDomain(Utils.UnobscureEmail(SLASH.join(parts[1:])))
+    # If a user submits a form or URL with post data or query fragments
+    # with multiple occurrences of the same variable, we can get a list
+    # here.  Be as careful as possible.
+    if isinstance(user, list) or isinstance(user, tuple):
+        if len(user) == 0:
+            user = ''
+        else:
+            user = user[-1]
 
     # Avoid cross-site scripting attacks
     safeuser = Utils.websafe(user)
@@ -138,12 +194,138 @@ def main():
     # And now we know the user making the request, so set things up to for the
     # user's stored preferred language, overridden by any form settings for
     # their new language preference.
-    userlang = cgidata.getvalue('language')
+    userlang = cgidata.getfirst('language')
     if not Utils.IsLanguage(userlang):
         userlang = mlist.getMemberLanguage(user)
     doc.set_language(userlang)
     i18n.set_language(userlang)
 
+    # Are we processing an unsubscription request from the login screen?
+    msgc = _('If you are a list member, a confirmation email has been sent.')
+    msga = _("""If you are a list member, your unsubscription request has been
+             forwarded to the list administrator for approval.""")
+    if cgidata.has_key('login-unsub'):
+        # Because they can't supply a password for unsubscribing, we'll need
+        # to do the confirmation dance.
+        if mlist.isMember(user):
+            # We must acquire the list lock in order to pend a request.
+            try:
+                mlist.Lock()
+                # If unsubs require admin approval, then this request has to
+                # be held.  Otherwise, send a confirmation.
+                if mlist.unsubscribe_policy:
+                    mlist.HoldUnsubscription(user)
+                    doc.addError(msga, tag='')
+                else:
+                    ip = os.environ.get('HTTP_FORWARDED_FOR',
+                         os.environ.get('HTTP_X_FORWARDED_FOR',
+                         os.environ.get('REMOTE_ADDR',
+                                        'unidentified origin')))
+                    mlist.ConfirmUnsubscription(user, userlang, remote=ip)
+                    doc.addError(msgc, tag='')
+                mlist.Save()
+            finally:
+                mlist.Unlock()
+        else:
+            # Not a member
+            if mlist.private_roster == 0:
+                # Public rosters
+                doc.addError(_('No such member: %(safeuser)s.'))
+            else:
+                syslog('mischief',
+                       'Unsub attempt of non-member w/ private rosters: %s',
+                       user)
+                if mlist.unsubscribe_policy:
+                    doc.addError(msga, tag='')
+                else:
+                    doc.addError(msgc, tag='')
+        loginpage(mlist, doc, user, language)
+        print doc.Format()
+        return
+
+    # Are we processing a password reminder from the login screen?
+    msg = _("""If you are a list member,
+            your password has been emailed to you.""")
+    if cgidata.has_key('login-remind'):
+        if mlist.isMember(user):
+            mlist.MailUserPassword(user)
+            doc.addError(msg, tag='')
+        else:
+            # Not a member
+            if mlist.private_roster == 0:
+                # Public rosters
+                doc.addError(_('No such member: %(safeuser)s.'))
+            else:
+                syslog('mischief',
+                       'Reminder attempt of non-member w/ private rosters: %s',
+                       user)
+                doc.addError(msg, tag='')
+        loginpage(mlist, doc, user, language)
+        print doc.Format()
+        return
+
+    # Get the password from the form.
+    password = cgidata.getfirst('password', '').strip()
+    # Check authentication.  We need to know if the credentials match the user
+    # or the site admin, because they are the only ones who are allowed to
+    # change things globally.  Specifically, the list admin may not change
+    # values globally.
+    if mm_cfg.ALLOW_SITE_ADMIN_COOKIES:
+        user_or_siteadmin_context = (mm_cfg.AuthUser, mm_cfg.AuthSiteAdmin)
+    else:
+        # Site and list admins are treated equal so that list admin can pass
+        # site admin test. :-(
+        user_or_siteadmin_context = (mm_cfg.AuthUser,)
+    is_user_or_siteadmin = mlist.WebAuthenticate(
+        user_or_siteadmin_context, password, user)
+    # Authenticate, possibly using the password supplied in the login page
+    if not is_user_or_siteadmin and \
+       not mlist.WebAuthenticate((mm_cfg.AuthListAdmin,
+                                  mm_cfg.AuthSiteAdmin),
+                                 password, user):
+        # Not authenticated, so throw up the login page again.  If they tried
+        # to authenticate via cgi (instead of cookie), then print an error
+        # message.
+        if cgidata.has_key('password'):
+            doc.addError(_('Authentication failed.'))
+            # So as not to allow membership leakage, prompt for the email
+            # address and the password here.
+            if mlist.private_roster <> 0:
+                remote = os.environ.get('HTTP_FORWARDED_FOR',
+                         os.environ.get('HTTP_X_FORWARDED_FOR',
+                         os.environ.get('REMOTE_ADDR',
+                                        'unidentified origin')))
+                syslog('mischief',
+                       'Login failure with private rosters: %s from %s',
+                       user, remote)
+                user = None
+            # give an HTTP 401 for authentication failure
+            print 'Status: 401 Unauthorized'
+        loginpage(mlist, doc, user, language)
+        print doc.Format()
+        return
+
+    # From here on out, the user is okay to view and modify their membership
+    # options.  The first set of checks does not require the list to be
+    # locked.
+
+    # However, if a form is submitted for a user who has been asynchronously
+    # unsubscribed, uncaught NotAMemberError exceptions can be thrown.
+
+    if not mlist.isMember(user):
+        loginpage(mlist, doc, user, language)
+        print doc.Format()
+        return
+
+    # Before going further, get the result of CSRF check and do nothing 
+    # if it has failed.
+    if csrf_checked == False:
+        doc.addError(
+            _('The form lifetime has expired. (request forgery check)'))
+        options_page(mlist, doc, user, cpuser, userlang)
+        print doc.Format()
+        return
+
     # See if this is VARHELP on topics.
     varhelp = None
     if cgidata.has_key('VARHELP'):
@@ -155,95 +337,11 @@ def main():
         if qs and type(qs) == types.ListType:
             varhelp = qs[0]
     if varhelp:
+        # Sanitize the topic name.
+        varhelp = re.sub('<.*', '', varhelp)
         topic_details(mlist, doc, user, cpuser, userlang, varhelp)
         return
 
-    # Are we processing an unsubscription request from the login screen?
-    if cgidata.has_key('login-unsub'):
-        # Because they can't supply a password for unsubscribing, we'll need
-        # to do the confirmation dance.
-        if mlist.isMember(user):
-            # If unsubs require admin approval, then this request has to be
-            # held.  Otherwise, send a confirmation.
-            if mlist.unsubscribe_policy:
-                try:
-                    mlist.Lock()
-                    mlist.HoldUnsubscription(user)
-                    doc.addError(_("""Your unsubscription request has been
-                    forwarded to the list administrator for approval."""),
-                                 tag='')
-                    mlist.Save()
-                finally:
-                    mlist.Unlock()
-            else:
-                mlist.ConfirmUnsubscription(user, userlang)
-                doc.addError(_('The confirmation email has been sent.'),
-                             tag='')
-        else:
-            # Not a member
-            if mlist.private_roster == 0:
-                # Public rosters
-                doc.addError(_('No such member: %(safeuser)s.'))
-            else:
-                syslog('mischief',
-                       'Unsub attempt of non-member w/ private rosters: %s',
-                       user)
-                doc.addError(_('The confirmation email has been sent.'),
-                             tag='')
-        loginpage(mlist, doc, user, language)
-        print doc.Format()
-        return
-
-    # Are we processing a password reminder from the login screen?
-    if cgidata.has_key('login-remind'):
-        if mlist.isMember(user):
-            mlist.MailUserPassword(user)
-            doc.addError(
-                _('A reminder of your password has been emailed to you.'),
-                tag='')
-        else:
-            # Not a member
-            if mlist.private_roster == 0:
-                # Public rosters
-                doc.addError(_('No such member: %(safeuser)s.'))
-            else:
-                syslog('mischief',
-                       'Reminder attempt of non-member w/ private rosters: %s',
-                       user)
-                doc.addError(
-                    _('A reminder of your password has been emailed to you.'),
-                    tag='')
-        loginpage(mlist, doc, user, language)
-        print doc.Format()
-        return
-
-    # Authenticate, possibly using the password supplied in the login page
-    password = cgidata.getvalue('password', '').strip()
-
-    if not mlist.WebAuthenticate((mm_cfg.AuthUser,
-                                  mm_cfg.AuthListAdmin,
-                                  mm_cfg.AuthSiteAdmin),
-                                 password, user):
-        # Not authenticated, so throw up the login page again.  If they tried
-        # to authenticate via cgi (instead of cookie), then print an error
-        # message.
-        if cgidata.has_key('password'):
-            doc.addError(_('Authentication failed.'))
-            # So as not to allow membership leakage, prompt for the email
-            # address and the password here.
-            if mlist.private_roster <> 0:
-                syslog('mischief',
-                       'Login failure with private rosters: %s',
-                       user)
-                user = None
-        loginpage(mlist, doc, user, language)
-        print doc.Format()
-        return
-
-    # From here on out, the user is okay to view and modify their membership
-    # options.  The first set of checks does not require the list to be
-    # locked.
-
     if cgidata.has_key('logout'):
         print mlist.ZapCookie(mm_cfg.AuthUser, user)
         loginpage(mlist, doc, user, language)
@@ -259,6 +357,13 @@ def main():
         return
 
     if cgidata.has_key('othersubs'):
+        # Only the user or site administrator can view all subscriptions.
+        if not is_user_or_siteadmin:
+            doc.addError(_("""The list administrator may not view the other
+            subscriptions for this user."""), _('Note: '))
+            options_page(mlist, doc, user, cpuser, userlang)
+            print doc.Format()
+            return
         hostname = mlist.host_name
         title = _('List subscriptions for %(safeuser)s on %(hostname)s')
         doc.SetTitle(title)
@@ -270,8 +375,14 @@ def main():
         # the user is a member.  If so, add it to the list.
         onlists = []
         for gmlist in lists_of_member(mlist, user) + [mlist]:
+            extra = ''
             url = gmlist.GetOptionsURL(user)
             link = Link(url, gmlist.real_name)
+            if gmlist.getDeliveryStatus(user) <> MemberAdaptor.ENABLED:
+                extra += ', ' + _('nomail')
+            if user in gmlist.getDigestMemberKeys():
+                extra += ', ' + _('digest')
+            link = HTMLFormatObject(link, 0) + extra
             onlists.append((gmlist.real_name, link))
         onlists.sort()
         items = OrderedList(*[link for name, link in onlists])
@@ -282,18 +393,24 @@ def main():
     if cgidata.has_key('change-of-address'):
         # We could be changing the user's full name, email address, or both.
         # Watch out for non-ASCII characters in the member's name.
-        membername = cgidata.getvalue('fullname')
+        membername = cgidata.getfirst('fullname')
         # Canonicalize the member's name
         membername = Utils.canonstr(membername, language)
-        newaddr = cgidata.getvalue('new-address')
-        confirmaddr = cgidata.getvalue('confirm-address')
+        newaddr = cgidata.getfirst('new-address')
+        confirmaddr = cgidata.getfirst('confirm-address')
 
         oldname = mlist.getMemberName(user)
         set_address = set_membername = 0
 
-        # See if the user wants to change their email address globally
-        globally = cgidata.getvalue('changeaddr-globally')
-
+        # See if the user wants to change their email address globally.  The
+        # list admin is /not/ allowed to make global changes.
+        globally = cgidata.getfirst('changeaddr-globally')
+        if globally and not is_user_or_siteadmin:
+            doc.addError(_("""The list administrator may not change the names
+            or addresses for this user's other subscriptions.  However, the
+            subscription for this mailing list has been changed."""),
+                         _('Note: '))
+            globally = False
         # We will change the member's name under the following conditions:
         # - membername has a value
         # - membername has no value, but they /used/ to have a membername
@@ -372,6 +489,11 @@ address.  Upon confirmation, any other mailing list containing the address
                 msg = _('Illegal email address provided')
             except Errors.MMAlreadyAMember:
                 msg = _('%(newaddr)s is already a member of the list.')
+            except Errors.MembershipIsBanned:
+                owneraddr = mlist.GetOwnerEmail()
+                msg = _("""%(newaddr)s is banned from this list.  If you
+                      think this restriction is erroneous, please contact
+                      the list owners at %(owneraddr)s.""")
 
         if set_membername:
             mlist.Lock()
@@ -387,8 +509,16 @@ address.  Upon confirmation, any other mailing list containing the address
         return
 
     if cgidata.has_key('changepw'):
-        newpw = cgidata.getvalue('newpw')
-        confirmpw = cgidata.getvalue('confpw')
+        # Is this list admin and is list admin allowed to change passwords.
+        if not (is_user_or_siteadmin
+                or mm_cfg.OWNERS_CAN_CHANGE_MEMBER_PASSWORDS):
+            doc.addError(_("""The list administrator may not change the
+                    password for a user."""))
+            options_page(mlist, doc, user, cpuser, userlang)
+            print doc.Format()
+            return
+        newpw = cgidata.getfirst('newpw', '').strip()
+        confirmpw = cgidata.getfirst('confpw', '').strip()
         if not newpw or not confirmpw:
             options_page(mlist, doc, user, cpuser, userlang,
                          _('Passwords may not be blank'))
@@ -400,9 +530,19 @@ address.  Upon confirmation, any other mailing list containing the address
             print doc.Format()
             return
 
-        # See if the user wants to change their passwords globally
+        # See if the user wants to change their passwords globally, however
+        # the list admin is /not/ allowed to change passwords globally.
+        pw_globally = cgidata.getfirst('pw-globally')
+        if pw_globally and not is_user_or_siteadmin:
+            doc.addError(_("""The list administrator may not change the
+            password for this user's other subscriptions.  However, the
+            password for this mailing list has been changed."""),
+                         _('Note: '))
+            pw_globally = False
+
         mlists = [mlist]
-        if cgidata.getvalue('pw-globally'):
+
+        if pw_globally:
             mlists.extend(lists_of_member(mlist, user))
 
         for gmlist in mlists:
@@ -417,7 +557,7 @@ address.  Upon confirmation, any other mailing list containing the address
 
     if cgidata.has_key('unsub'):
         # Was the confirming check box turned on?
-        if not cgidata.getvalue('unsubconfirm'):
+        if not cgidata.getfirst('unsubconfirm'):
             options_page(
                 mlist, doc, user, cpuser, userlang,
                 _('''You must confirm your unsubscription request by turning
@@ -443,6 +583,13 @@ address.  Upon confirmation, any other mailing list containing the address
                     user, 'via the member options page', userack=1)
             except Errors.MMNeedApproval:
                 needapproval = True
+            except Errors.NotAMemberError:
+                # MAS This except should really be in the outer try so we
+                # don't save the list redundantly, but except and finally in
+                # the same try requires Python >= 2.5.
+                # Setting a switch and making the Save() conditional doesn't
+                # seem worth it as the Save() won't change anything.
+                pass
             mlist.Save()
         finally:
             mlist.Unlock()
@@ -490,7 +637,7 @@ address.  Upon confirmation, any other mailing list containing the address
                            ('nodupes',     mm_cfg.DontReceiveDuplicates),
                            ):
             try:
-                newval = int(cgidata.getvalue(item))
+                newval = int(cgidata.getfirst(item))
             except (TypeError, ValueError):
                 newval = None
 
@@ -580,7 +727,7 @@ address.  Upon confirmation, any other mailing list containing the address
 
         # The enable/disable option and the password remind option may have
         # their global flags sets.
-        if cgidata.getvalue('deliver-globally'):
+        if cgidata.getfirst('deliver-globally'):
             # Yes, this is inefficient, but the list is so small it shouldn't
             # make much of a difference.
             for flag, newval in newvals:
@@ -588,27 +735,35 @@ address.  Upon confirmation, any other mailing list containing the address
                     globalopts.enable = newval
                     break
 
-        if cgidata.getvalue('remind-globally'):
+        if cgidata.getfirst('remind-globally'):
             for flag, newval in newvals:
                 if flag == mm_cfg.SuppressPasswordReminder:
                     globalopts.remind = newval
                     break
 
-        if cgidata.getvalue('nodupes-globally'):
+        if cgidata.getfirst('nodupes-globally'):
             for flag, newval in newvals:
                 if flag == mm_cfg.DontReceiveDuplicates:
                     globalopts.nodupes = newval
                     break
 
-        if cgidata.getvalue('mime-globally'):
+        if cgidata.getfirst('mime-globally'):
             for flag, newval in newvals:
                 if flag == mm_cfg.DisableMime:
                     globalopts.mime = newval
                     break
 
+        # Change options globally, but only if this is the user or site admin,
+        # /not/ if this is the list admin.
         if globalopts:
-            for gmlist in lists_of_member(mlist, user):
-                global_options(gmlist, user, globalopts)
+            if not is_user_or_siteadmin:
+                doc.addError(_("""The list administrator may not change the
+                options for this user's other subscriptions.  However the
+                options for this mailing list subscription has been
+                changed."""), _('Note: '))
+            else:
+                for gmlist in lists_of_member(mlist, user):
+                    global_options(gmlist, user, globalopts)
 
         # Now print the results
         if cantdigest:
@@ -629,7 +784,10 @@ address.  Upon confirmation, any other mailing list containing the address
         print doc.Format()
         return
 
-    options_page(mlist, doc, user, cpuser, userlang)
+    if mlist.isMember(user):
+        options_page(mlist, doc, user, cpuser, userlang)
+    else:
+        loginpage(mlist, doc, user, userlang)
     print doc.Format()
 
 
@@ -649,7 +807,7 @@ def options_page(mlist, doc, user, cpuser, userlang, message=''):
 
     fullname = Utils.uncanonstr(mlist.getMemberName(user), userlang)
     if fullname:
-        presentable_user += ', %s' % fullname
+        presentable_user += ', %s' % Utils.websafe(fullname)
 
     # Do replacements
     replacements = mlist.GetStandardReplacements(userlang)
@@ -701,7 +859,8 @@ def options_page(mlist, doc, user, cpuser, userlang, message=''):
         mlist.FormatButton('othersubs',
                            _('List my other subscriptions')))
     replacements['<mm-form-start>'] = (
-        mlist.FormatFormStart('options', user))
+        mlist.FormatFormStart('options', user, mlist=mlist, 
+            contexts=AUTH_CONTEXTS, user=user))
     replacements['<mm-user>'] = user
     replacements['<mm-presentable-user>'] = presentable_user
     replacements['<mm-email-my-pw>'] = mlist.FormatButton(
@@ -744,6 +903,8 @@ def options_page(mlist, doc, user, cpuser, userlang, message=''):
     if mlist.topics:
         table = Table(border="0")
         for name, pattern, description, emptyflag in mlist.topics:
+            if emptyflag:
+                continue
             quotedname = urllib.quote_plus(name)
             details = Link(mlist.GetScriptURL('options') +
                            '/%s/?VARHELP=%s' % (user, quotedname),
@@ -770,8 +931,10 @@ You are subscribed to this list with the case-preserved address
     else:
         replacements['<mm-case-preserved-user>'] = ''
 
-    doc.AddItem(mlist.ParseTags('options.html', replacements, userlang))
-
+    page_text = mlist.ParseTags('options.html', replacements, userlang)
+    if not (mlist.digestable or mlist.getMemberOption(user, mm_cfg.Digests)):
+        page_text = DIGRE.sub('', page_text)
+    doc.AddItem(page_text)
 
 
 def loginpage(mlist, doc, user, lang):
@@ -806,6 +969,7 @@ def loginpage(mlist, doc, user, lang):
     # Preamble
     # Set up the login page
     form = Form(actionurl)
+    form.AddItem(Hidden('language', lang))
     table = Table(width='100%', border=0, cellspacing=4, cellpadding=5)
     table.AddRow([_("""In order to change your membership option, you must
     first log in by giving your %(extra)smembership password in the section
@@ -972,7 +1136,8 @@ def topic_details(mlist, doc, user, cpuser, userlang, varhelp):
     table.AddRow([Bold(Label(_('Name:'))),
                   Utils.websafe(name)])
     table.AddRow([Bold(Label(_('Pattern (as regexp):'))),
-                  '<pre>' + Utils.websafe(pattern) + '</pre>'])
+                  '<pre>' + Utils.websafe(OR.join(pattern.splitlines()))
+                   + '</pre>'])
     table.AddRow([Bold(Label(_('Description:'))),
                   Utils.websafe(description)])
     # Make colors look nice
