@@ -1,6 +1,6 @@
 # -*- python -*-
 
-# Copyright (C) 1998-2016 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2018 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -131,6 +131,12 @@ SUBSCRIBE_FORM_SECRET = None
 # test.
 SUBSCRIBE_FORM_MIN_TIME = seconds(5)
 
+# Use Google reCAPTCHA to protect the subscription form from spam bots.  The
+# following must be set to a pair of keys issued by the reCAPTCHA service at
+# https://www.google.com/recaptcha/admin
+RECAPTCHA_SITE_KEY = None
+RECAPTCHA_SECRET_KEY = None
+
 # Installation wide ban list.  This is a list of email addresses and regexp
 # patterns (beginning with ^) which are not allowed to subscribe to any lists
 # in the installation.  This supplements the individual list's ban_list.
@@ -185,7 +191,7 @@ VIRTUAL_HOST_OVERVIEW = On
 # omitted it defaults to urlhost with the first name stripped off, e.g.
 #
 # add_virtualhost('www.dom.ain')
-# VIRTUAL_HOST['www.dom.ain']
+# VIRTUAL_HOSTS['www.dom.ain']
 # ==> 'dom.ain'
 #
 def add_virtualhost(urlhost, emailhost=None):
@@ -629,6 +635,17 @@ NNTP_REWRITE_DUPLICATE_HEADERS = [
 # 3 -> always remove, rename and preserve original DKIM headers.
 REMOVE_DKIM_HEADERS = No
 
+# If the following is set to a non-empty string, that string is the name of a
+# header that will be added to personalized and VERPed deliveries with value
+# equal to the base64 encoding of the recipient's email address.  This is
+# intended to enable identification of the recipient otherwise redacted from
+# "spam report" feedback loop messages.  For example, if
+# RCPT_BASE64_HEADER_NAME = 'X-Mailman-R-Data'
+# a header like
+# X-Mailman-R-Data: dXNlckBleGFtcGxlLmNvbQo=
+# will be added to messages sent to user@@example.com.
+RCPT_BASE64_HEADER_NAME = ''
+
 # All `normal' messages which are delivered to the entire list membership go
 # through this pipeline of handler modules.  Lists themselves can override the
 # global pipeline by defining a `pipeline' attribute.
@@ -666,7 +683,6 @@ GLOBAL_PIPELINE = [
 OWNER_PIPELINE = [
     'SpamDetect',
     'Replybot',
-    'CleanseDKIM',
     'OwnerRecips',
     'ToOutgoing',
     ]
@@ -1088,7 +1104,7 @@ DEFAULT_MAX_MESSAGE_SIZE = 40           # KB
 DEFAULT_SUBJECT_PREFIX  = "[%(real_name)s] "
 # DEFAULT_SUBJECT_PREFIX = "[%(real_name)s %%d]" # for numbering
 DEFAULT_MSG_HEADER = ""
-DEFAULT_MSG_FOOTER = """_______________________________________________
+DEFAULT_MSG_FOOTER = """-- 
 %(real_name)s mailing list
 %(real_name)s@%(host_name)s
 %(web_page_url)slistinfo%(cgiext)s/%(_internal_name)s
@@ -1398,7 +1414,10 @@ DEFAULT_NONDIGESTABLE = Yes
 # Will list be available in digested form?
 DEFAULT_DIGESTABLE = Yes
 DEFAULT_DIGEST_HEADER = ""
-DEFAULT_DIGEST_FOOTER = DEFAULT_MSG_FOOTER
+DEFAULT_DIGEST_FOOTER = """%(real_name)s mailing list
+%(real_name)s@%(host_name)s
+%(web_page_url)slistinfo%(cgiext)s/%(_internal_name)s
+"""
 
 DEFAULT_DIGEST_IS_DEFAULT = No
 DEFAULT_MIME_IS_DEFAULT_DIGEST = No
@@ -1509,6 +1528,9 @@ PENDING_REQUEST_LIFE = days(3)
 DELIVERY_RETRY_PERIOD = days(5)
 
 # How long should we wait before we retry a temporary delivery failure?
+# Because RetryRunner sleeps for 15 minutes between processes of its queue,
+# whatever is put here is effectively rounded up to the next integer multiple
+# of 15 minutes.
 DELIVERY_RETRY_WAIT = hours(1)
 
 
