@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# Copyright (C) 2002 by the Free Software Foundation, Inc.
+# Copyright (C) 2002-2013 by the Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -14,7 +14,8 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
+# USA.
 
 """Test the bounce detection for files containing bounces.
 
@@ -80,6 +81,7 @@ def main():
                 print module, 'got a Stop'
                 if not all:
                     break
+                continue
             if not addrs:
                 if verbose:
                     print module, 'found no matches'
